import os
import json

from oct.core.exceptions import OctConfigurationError


REQUIRED_CONFIG_KEYS = [
    'run_time',
    'results_ts_interval',
    'testing',
    'progress_bar',
    'turrets'
]

WARNING_CONFIG_KEYS = [
    'hq_address',
    'publish_port',
    'rc_port'
]

REMOVABLE_KEYS = [
    'turrets_requirements'
]


def configure(project_name, cmd_opts, config_file=None):
    """Get the configuration of the test and return it as a config object

    :return: the configured config object
    :rtype: Object
    """
    if config_file is None:
        config_file = os.path.join(cmd_opts.project_dir, project_name, 'config.json')
    try:
        with open(config_file, 'r') as f:
            config = json.load(f)
    except ValueError as e:
        raise OctConfigurationError("Configuration setting failed with error: %s" % e)
    for key in REQUIRED_CONFIG_KEYS:
        if key not in config:
            raise OctConfigurationError("Error: the required configuration key %s is not define" % key)
    return config


def configure_for_turret(project_name, config_file):
    """Load the configuration file in python dict and check for keys that will be set to default value if not present

    :param str project_name: the name of the project
    :param str config_file: the path of the configuration file
    :return: the loaded configuration
    :rtype: dict
    """
    config = configure(project_name, None, config_file)
    for key in WARNING_CONFIG_KEYS:
        if key not in config:
            print("WARNING: %s configuration key not present, the value will be set to default value" % key)
    common_config = {
        'hq_address': config.get('hq_address', '127.0.0.1'),
        'hq_publisher': config.get('publish_port', 5000),
        'hq_rc': config.get('rc_port', 5001),
        'turrets_requirements': config.get('turrets_requirements', [])
    }
    configs = []
    for turret in config['turrets']:
        turret.update(common_config)
        turret.update(config.get('extra_turret_config', {}))
        configs.append(turret)
    return configs


def cleanup_turret_config(config):
    """Remove useless keys from turret configuration

    :param dict config: the configuration to cleanup
    :return: the cleaned configuration
    :rtype: dict
    """
    for key in REMOVABLE_KEYS:
        if key in config:
            del config[key]

    return config



import six
import time
from collections import defaultdict

import ujson as json
import pandas as pd

from oct.results.models import db, Result, Turret


class ReportResults(object):
    """Represent a report containing all tests results

    :param int run_time: the run_time of the script
    :param int interval: the time interval between each group of results
    """
    def __init__(self, run_time, interval):
        self.total_transactions = 0
        self.total_errors = Result.select(Result.id).where(Result.error != "", Result.error != None).count()
        self.total_timers = 0
        self.timers_results = {}
        self._timers_values = defaultdict(list)
        self.turrets = []
        self.main_results = {}
        self.interval = interval

        self._init_turrets()

    def _init_dates(self):
        """Initialize all dates properties
        """
        if self.total_transactions == 0:
            return None
        self.epoch_start = Result.select(Result.epoch).order_by(Result.epoch.asc()).limit(1).get().epoch
        self.epoch_finish = Result.select(Result.epoch).order_by(Result.epoch.desc()).limit(1).get().epoch
        self.start_datetime = time.strftime('%Y-%m-%d %H:%M:%S', time.localtime(self.epoch_start))
        self.finish_datetime = time.strftime('%Y-%m-%d %H:%M:%S', time.localtime(self.epoch_finish))

    def _init_dataframes(self):
        """Initialise the main dataframe for the results and the custom timers dataframes
        """
        df = pd.read_sql_query("SELECT elapsed, epoch, scriptrun_time, custom_timers FROM result ORDER BY epoch ASC",
                               db.get_conn())

        self._get_all_timers(df)
        self.main_results = self._get_processed_dataframe(df)

        # create all custom timers dataframes
        for key, value in six.iteritems(self._timers_values):
            df = pd.DataFrame(value, columns=['epoch', 'scriptrun_time'])
            df.index = pd.to_datetime(df['epoch'], unit='s')
            timer_results = self._get_processed_dataframe(df)
            self.timers_results[key] = timer_results

        # clear memory
        del self._timers_values

    def _get_all_timers(self, dataframe):
        """Get all timers and set them in the _timers_values property

        :param pandas.DataFrame dataframe: the main dataframe with row results
        """
        s = dataframe['custom_timers'].apply(json.loads)
        s.index = dataframe['epoch']
        for index, value in s.iteritems():
            if not value:
                continue
            for key, value in six.iteritems(value):
                self._timers_values[key].append((index, value))
                self.total_timers += 1
        del dataframe['custom_timers']
        del s

    def _get_processed_dataframe(self, dataframe):
        """Generate required dataframe for results from raw dataframe

        :param pandas.DataFrame dataframe: the raw dataframe
        :return: a dict containing raw, compiled, and summary dataframes from original dataframe
        :rtype: dict
        """
        dataframe.index = pd.to_datetime(dataframe['epoch'], unit='s', utc=True)
        del dataframe['epoch']
        summary = dataframe.describe(percentiles=[.80, .90, .95]).transpose().loc['scriptrun_time']
        df_grp = dataframe.groupby(pd.TimeGrouper('{}S'.format(self.interval)))
        df_final = df_grp.apply(lambda x: x.describe(percentiles=[.80, .90, .95])['scriptrun_time'])

        return {
            "raw": dataframe.round(2),
            "compiled": df_final.round(2),
            "summary": summary.round(2)
        }

    def _init_turrets(self):
        """Setup data from database
        """
        for turret in Turret.select():
            self.turrets.append(turret.to_dict())

    def compile_results(self):
        """Compile all results for the current test
        """
        self._init_dataframes()

        self.total_transactions = len(self.main_results['raw'])
        self._init_dates()



from __future__ import print_function
import argparse

from oct.utilities.newproject import new_project
from oct.utilities.pack import pack_turrets
from oct.utilities.run import run_command
from oct.tools.rebuild_results import rebuild_results
from oct.tools.results_to_csv import results_to_csv


PARSERS = [
    new_project,
    pack_turrets,
    run_command,
    rebuild_results,
    results_to_csv
]


def build_parser():
    parser = argparse.ArgumentParser(prog='oct')
    subparsers = parser.add_subparsers(help='sub commands avaibles')

    for p in PARSERS:
        p(subparsers)

    return parser


def main():
    parser = build_parser()
    args = parser.parse_args()
    args.func(args)



import os
import shutil


class ReportWriter(object):
    """A class representing a report, used to output the result

    :param str results_dir: the output directory for the report
    :param str parent: the parent directory
    """
    def __init__(self, results_dir, parent):
        self.results_dir = results_dir
        self.fn = os.path.join(results_dir, 'results.html')
        self.templates_dir = os.path.join(results_dir, '../../', 'templates')

        self.set_statics()

    def set_statics(self):
        """Create statics directory and copy files in it
        """
        if not os.path.exists(self.results_dir):
            return None
        try:
            shutil.copytree(os.path.join(self.templates_dir, 'css'), os.path.join(self.results_dir, 'css'))
            shutil.copytree(os.path.join(self.templates_dir, 'img'), os.path.join(self.results_dir, 'img'))
            shutil.copytree(os.path.join(self.templates_dir, 'scripts'), os.path.join(self.results_dir, 'scripts'))
            shutil.copytree(os.path.join(self.templates_dir, 'fonts'), os.path.join(self.results_dir, 'fonts'))
        except OSError as e:
            if e.errno == 17:  # File exists
                print("WARNING : existing output directory for static files, will not replace them")
            else:  # in all other cases, re-raise exceptions
                raise

    def write_report(self, template):
        """Write the compiled jinja template to the results file

        :param template str: the compiled jinja template
        """
        with open(self.fn, 'w') as f:
            f.write(template)



from __future__ import print_function

import os
import time
from jinja2 import Environment, FileSystemLoader

from oct.results import graphs
from oct.results.report import ReportResults
from oct.results.writer import ReportWriter


def generate_graphs(data, name, results_dir):
    """Generate all reports from original dataframe

    :param dic data: dict containing raw and compiled results dataframes
    :param str name: name for prefixing graphs output
    :param str results_dir: results output directory
    """
    graphs.resp_graph_raw(data['raw'], name + '_response_times.svg', results_dir)
    graphs.resp_graph(data['compiled'], name + '_response_times_intervals.svg', results_dir)
    graphs.tp_graph(data['compiled'], name + '_throughput.svg', results_dir)


def print_infos(results):
    """Print informations in standard output

    :param ReportResults results: the report result containing all compiled informations
    """
    print('transactions: %i' % results.total_transactions)
    print('timers: %i' % results.total_timers)
    print('errors: %i' % results.total_errors)
    print('test start: %s' % results.start_datetime)
    print('test finish: %s\n' % results.finish_datetime)


def write_template(data, results_dir, parent):
    """Write the html template

    :param dict data: the dict containing all data for output
    :param str results_dir: the ouput directory for results
    :param str parent: the parent directory
    """
    print("Generating html report...")
    partial = time.time()
    j_env = Environment(loader=FileSystemLoader(os.path.join(results_dir, parent, 'templates')))
    template = j_env.get_template('report.html')

    report_writer = ReportWriter(results_dir, parent)
    report_writer.write_report(template.render(data))
    print("HTML report generated in {} seconds\n".format(time.time() - partial))


def output(results_dir, config, parent='../../'):
    """Write the results output for the given test

    :param str results_dir: the directory for the results
    :param dict config: the configuration of the test
    :param str parents: the parent directory
    """
    start = time.time()
    print("Compiling results...")
    results_dir = os.path.abspath(results_dir)
    results = ReportResults(config['run_time'], config['results_ts_interval'])
    results.compile_results()
    print("Results compiled in {} seconds\n".format(time.time() - start))

    if results.total_transactions == 0:
        print("No results, cannot create report")
        return False

    print_infos(results)

    data = {
        'report': results,
        'run_time': config['run_time'],
        'ts_interval': config['results_ts_interval'],
        'turrets_config': results.turrets,
        'results': {"all": results.main_results, "timers": results.timers_results}
    }

    print("Generating graphs...")
    partial = time.time()
    generate_graphs(results.main_results, 'All_Transactions', results_dir)

    for key, value in results.timers_results.items():
        generate_graphs(value, key, results_dir)
    print("All graphs generated in {} seconds\n".format(time.time() - partial))

    write_template(data, results_dir, parent)
    print("Full report generated in {} seconds".format(time.time() - start))
    return True



import sys
import six
import json
import tarfile
import os.path
import tempfile
from jinja2 import Environment, PackageLoader

from oct.utilities.configuration import configure_for_turret, cleanup_turret_config


def cleanup_temp_files(turret, files):
    for f in files:
        try:
            os.remove(f)
        except IOError as e:
            print("Error while packagin turret %s" % turret['name'])
            print("Error: %s" % e)


def write_temp_files(turret, files):
    writed_files = []
    for f in files:
        try:
            with open(f['filename'], 'w') as fd:
                fd.write(f['content'])
                writed_files.append(f['filename'])
        except IOError as e:
            print("Error while packaging turret %s" % turret['name'])
            print("Error: %s" % e)
    return writed_files


def get_files_and_content(turret, is_python=False):
    ret = []
    env = Environment(loader=PackageLoader('oct.utilities', 'templates'))
    turret_setup = env.get_template('scripts/setup.j2')

    if is_python:
        tmp_setup = os.path.join(tempfile.gettempdir(), "setup.py")
        turrets_requirements = turret.get('turrets_requirements', [])
        setup_file = turret_setup.render({'turrets_requirements': turrets_requirements,
                                         'name': turret.get('name')})
        ret.append({'filename': tmp_setup, 'content': setup_file})

    content = json.dumps(cleanup_turret_config(turret), indent=2)
    tmp_config = os.path.join(tempfile.gettempdir(), "config.json")
    ret.append({'filename': tmp_config, 'content': content})
    return ret


def pack_turret(turret, temp_files, base_config_path, path=None):
    """pack a turret into a tar file based on the turret configuration

    :param dict turret_config: the turret configuration to pack
    :param str tmp_config_file: the path of the temp config file
    :param str base_config_path: the base directory of the main configuration file
    """
    file_name = turret['name']
    files = temp_files[:]
    for fname in turret.get('extra_files', []):
        if os.path.isabs(fname) or path is None:
            files.append(fname)
        else:
            files.append(os.path.join(path, fname))
    if path is not None:
        file_name = os.path.join(path, file_name)
    tar_file = tarfile.open(file_name + ".tar.gz", 'w:gz')

    for f in files:
        tar_file.add(os.path.abspath(f), arcname=os.path.basename(f))

    script_path = os.path.join(os.path.abspath(base_config_path), turret['script'])
    tar_file.add(script_path, arcname=turret['script'])

    for f in tar_file.getnames():
        print("Added %s" % f)
    tar_file.close()
    print("Archive %s created" % (tar_file.name))
    print("=========================================")


def pack(args):
    if os.path.exists(args.path):
        config_file = os.path.join(os.path.abspath(args.path), "config.json")
        configs = configure_for_turret(args.path, config_file)

        for turret in configs:
            files = get_files_and_content(turret, args.python)
            temp_files = write_temp_files(turret, files)
            pack_turret(turret, temp_files, os.path.dirname(config_file), args.path)
            cleanup_temp_files(turret, temp_files)

    else:
        print("you need to enter a valid project path")
        sys.exit(2)


def pack_turrets(sp):
    if six.PY2:
        parser = sp.add_parser('pack-turrets',
                               help="create turrets packages from a given oct project")
    else:
        parser = sp.add_parser('pack-turrets',
                               help="create turrets packages from a given oct project",
                               aliases=['pack'])
    parser.add_argument('path', type=str, help='path for oct project dir')
    parser.add_argument('--python', action='store_true', default=False, help='If set, a setup.py file will be created')
    parser.set_defaults(func=pack)



import os
import ujson

from oct.results.models import Result, Turret, set_database, db


def init_stats(output_dir, config):
    """Init all required ressources for stats handling
    :param str output_dir: the output directory for the results
    :param dict config: the project configuration
    """

    try:
        os.makedirs(output_dir, 0o755)
    except OSError:
        print("ERROR: Can not create output directory\n")
        raise

    set_database(output_dir + "results.sqlite", db, config)
    db.connect()
    db.create_tables([Result, Turret])


class StatsHandler(object):
    """This class will handle results and stats comming from the turrets
    :param str output_dir: the output directory for the results
    """
    def __init__(self, output_dir, config, context=None):
        self.output_dir = output_dir
        self.turret_name = 'Turret'
        self.results = []

    def write_result(self, data):
        """Write the results received to the database
        :param dict data: the data to save in database
        :return: None
        """
        data['custom_timers'] = ujson.dumps(data['custom_timers'])
        self.results.append(data)

        if len(self.results) >= 490:  # SQLite limit for inser_many is 500
            with db.execution_context():
                with db.atomic():
                    Result.insert_many(self.results).execute()
                del self.results[:]

    def write_remaining(self):
        """Write the remaning stack content
        """
        if not self.results:
            return
        with db.execution_context():
            with db.atomic():
                Result.insert_many(self.results).execute()
        del self.results[:]



from __future__ import print_function

import zmq
import ujson as json

from oct.results.models import db, Turret


class TurretsManager(object):
    """Turrets management while runing test. This class is in charge to send
    message to turrets and to store informations about active turrets
    """
    STATUS_REQUEST = {'command': 'status_request', 'msg': None}
    START = {'command': 'start', 'msg': 'open fire'}
    STOP = {'command': 'stop', 'msg': 'premature stop'}

    def __init__(self, publish_port):
        self.turrets = {}

        context = zmq.Context()
        self.publisher = context.socket(zmq.PUB)
        self.publisher.bind("tcp://*:{}".format(publish_port))

    def clean(self):
        self.publisher.close()

    def start(self):
        """Publish start message to all turrets
        """
        self.publish(self.START)

    def stop(self):
        """Publish stop message to all turrets
        """
        self.publish(self.STOP)

    def status_request(self):
        """Publish a status request message
        """
        self.publish(self.STATUS_REQUEST)

    def process_message(self, message, is_started=False):
        """Process incomming message from turret

        :param dict message: incomming message
        :param bool is_started: test started indicator
        """
        if 'status' not in message:
            return False
        message['name'] = message['turret']
        del message['turret']
        if not self.add(message, is_started):
            return self.update(message)
        return True

    def add(self, turret_data, is_started=False):
        """Add a turret object to current turrets configuration

        :param dict turret_data: the data of the turret to add
        :param bool is_started: tell if test are already runing
        """
        if turret_data.get('uuid') in self.turrets:
            return False

        turret = Turret(**turret_data)
        self.write(turret)
        self.turrets[turret.uuid] = turret

        if is_started:
            self.publish(self.START, turret.uuid)

        return True

    def update(self, turret_data):
        """Update a given turret

        :param dict turret_data: the data of the turret to update
        """
        if turret_data.get('uuid') not in self.turrets:
            return False
        turret = self.turrets[turret_data.get('uuid')]
        turret.update(**turret_data)
        self.write(turret)
        return True

    def write(self, turret):
        """Write a turret to database

        :param dict turret_data: the data of the turret to write
        """
        with db.execution_context():
            turret.save()

    def publish(self, message, channel=None):
        """Publish a message for all turrets

        :param dict message: message to send to turrets
        :pram str channel: channel to send message, default to empty string
        """
        channel = channel or ''
        data = json.dumps(message)
        self.publisher.send_string("%s %s" % (channel, data))



from __future__ import print_function
import zmq
import time
import ujson
import traceback

from oct.core.turrets_manager import TurretsManager
from oct.results.stats_handler import StatsHandler


class HightQuarter(object):
    """The main hight quarter that will receive informations from the turrets
    and send the start message

    :param int publish_port: the port for publishing information to turrets
    :param int rc_port: the result collector port for collecting results from the turrets
    :param StatsHandler stats_handler: the stats handler writer
    :param dict config: the configuration of the test
    """
    def __init__(self, publish_port, rc_port, output_dir, config):
        self.context = zmq.Context()
        self.poller = zmq.Poller()

        self.result_collector = self.context.socket(zmq.PULL)
        self.result_collector.set_hwm(0)
        self.result_collector.bind("tcp://*:{}".format(rc_port))

        self.stats_handler = StatsHandler(output_dir, config)

        self.poller.register(self.result_collector, zmq.POLLIN)

        self.turrets_manager = TurretsManager(publish_port)
        self.config = config
        self.started = False
        self.messages = 0

        # waiting for init sockets
        print("Warmup")
        time.sleep(1)

    def _process_socks(self, socks):
        if self.result_collector in socks:
            self.messages += 1
            data = self.result_collector.recv()
            if b'status' not in data:
                self.stats_handler.write_result(ujson.loads(data))
                self.messages += 1
            else:
                self.turrets_manager.process_message(ujson.loads(data))
        pass

    def _print_status(self, elapsed):
        display = 'turrets: {}, elapsed: {}  messages received: {}\r'
        print(display.format(len(self.turrets_manager.turrets), round(elapsed), self.messages,), end='')

    def _clean_queue(self):
        try:
            data = self.result_collector.recv(zmq.NOBLOCK)
            while data:
                data = self.result_collector.recv(zmq.NOBLOCK)
                if b'status' not in data:
                    self.stats_handler.write_result(ujson.loads(data))
        except zmq.Again:
            self.result_collector.close()
            self.turrets_manager.clean()
            self.stats_handler.write_remaining()

    def _run_loop_action(self):
        socks = dict(self.poller.poll(1000))
        self._process_socks(socks)

    def wait_turrets(self, wait_for):
        """Wait until wait_for turrets are connected and ready
        """
        print("Waiting for %d turrets" % (wait_for - len(self.turrets_manager.turrets)))

        while len(self.turrets_manager.turrets) < wait_for:
            self.turrets_manager.status_request()

            socks = dict(self.poller.poll(2000))

            if self.result_collector in socks:
                data = self.result_collector.recv_json()
                self.turrets_manager.process_message(data)

                print("Waiting for %d turrets" % (wait_for - len(self.turrets_manager.turrets)))

    def run(self):
        """Run the hight quarter, lunch the turrets and wait for results
        """
        elapsed = 0
        run_time = self.config['run_time']
        start_time = time.time()
        t = time.time
        self.turrets_manager.start()
        self.started = True

        while elapsed <= run_time:
            try:
                self._run_loop_action()
                self._print_status(elapsed)
                elapsed = t() - start_time
            except (Exception, KeyboardInterrupt):
                print("\nStopping test, sending stop command to turrets")
                self.turrets_manager.stop()
                self.stats_handler.write_remaining()
                traceback.print_exc()
                break

        self.turrets_manager.stop()
        print("\n\nProcessing all remaining messages... This could take time depending on message volume")
        t = time.time()
        self.result_collector.unbind(self.result_collector.LAST_ENDPOINT)
        self._clean_queue()
        print("took %s" % (time.time() - t))



import os
from setuptools import setup
from oct import __version__

BASE_DIR = os.path.abspath(os.path.dirname(__file__))

with open('README.rst') as f:
    long_description = f.read()

setup(
    name='oct',
    version=__version__,
    author='TheGhouls',
    author_email='manu.valette@gmail.com',
    packages=['oct', 'oct.core', 'oct.utilities', 'oct.tools', 'oct.results'],
    package_data={
        'oct.utilities': [
            'templates/css/*',
            'templates/configuration/*',
            'templates/html/*',
            'templates/scripts/*',
            'templates/javascript/*',
            'templates/fonts/*'
        ]
    },
    description="A library for performances testing, give you the tools for load testing anything with any language",
    long_description=long_description,
    url='https://github.com/TheGhouls/oct',
    download_url='https://github.com/TheGhouls/oct/archive/master.zip',
    keywords=['testing', 'perfs', 'performances', 'web', 'load', 'zeromq'],
    classifiers=[
        'Development Status :: 4 - Beta',
        'Environment :: Console',
        'Intended Audience :: Developers',
        'Intended Audience :: System Administrators',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5'
    ],
    install_requires=[
        'pygal',
        'peewee',
        'oct-turrets>=0.2.4',
        'six',
        'pyzmq',
        'numpy',
        'jinja2',
        'pandas',
        'ujson'
    ],
    entry_points={'console_scripts': [
        'oct = oct.utilities.commands:main'
    ]},
)



import os
import pytz
import pygal
import numpy as np
import pandas as pd
from dateutil.tz import tzlocal


def get_local_time(index):
    """Localize datetime for better output in graphs

    :param pandas.DateTimeIndex index: pandas datetime index
    :return: aware time objet
    :rtype: datetime.time
    """
    dt = index.to_pydatetime()
    dt = dt.replace(tzinfo=pytz.utc)
    return dt.astimezone(tzlocal()).time()


def resp_graph_raw(dataframe, image_name, dir='./'):
    """Response time graph for raw data

    :param pandas.DataFrame dataframe: the raw results dataframe
    :param str image_name: the output file name
    :param str dir: the output directory
    :return: None
    """
    factor = int(len(dataframe) / 10)
    df = dataframe.reset_index()
    grp = df.groupby(pd.cut(df.index, np.arange(0, len(df), factor)))

    fig = pygal.Dot(stroke=False,
                    x_label_rotation=25,
                    x_title='Elapsed Time In Test (secs)',
                    y_title='Average Response Time (secs)',
                    js=('scripts/pygal-tooltip.min.js',))
    fig.x_labels = [x for x in grp.first()['epoch']]
    fig.title = image_name.split('.')[0]
    fig.add('Time', [x for x in grp.describe()['scriptrun_time'].unstack()['mean'].round(2)])
    fig.render_to_file(filename=os.path.join(dir, image_name))


def resp_graph(dataframe, image_name, dir='./'):
    """Response time graph for bucketed data

    :param pandas.DataFrame dataframe: dataframe containing all data
    :param str image_name: the output file name
    :param str dir: the output directory
    :return: None
    """
    fig = pygal.TimeLine(x_title='Elapsed Time In Test (secs)',
                         y_title='Response Time (secs)',
                         x_label_rotation=25,
                         js=('scripts/pygal-tooltip.min.js',))
    fig.add('AVG', [(get_local_time(index), row['mean'] if pd.notnull(row['mean']) else None)
                    for index, row in dataframe.iterrows()])
    fig.add('90%', [(get_local_time(index), row['90%'] if pd.notnull(row['90%']) else None)
                    for index, row in dataframe.iterrows()])
    fig.add('80%', [(get_local_time(index), row['80%'] if pd.notnull(row['80%']) else None)
                    for index, row in dataframe.iterrows()])
    fig.render_to_file(filename=os.path.join(dir, image_name))


def tp_graph(dataframe, image_name, dir='./'):
    """Throughput graph

    :param pandas.DataFrame dataframe: dataframe containing all data
    :param str dir: the output directory
    :return: None
    """
    fig = pygal.TimeLine(x_title='Elapsed Time In Test (secs)',
                         x_label_rotation=25,
                         y_title='Transactions Per Second (count)',
                         human_readable=True,
                         js=('scripts/pygal-tooltip.min.js',))
    fig.add('Transactions per second', [(get_local_time(index), row['count'])
                                        for index, row in dataframe.iterrows()])
    fig.render_to_file(filename=os.path.join(dir, image_name))



import six

from oct.results.output import output
from oct.results.models import db, set_database
from oct.utilities.configuration import configure


def rebuild(args):
    config = configure(None, None, args.config_file)
    set_database(args.results_file, db, config)
    output(args.results_dir, config)


def rebuild_results(sp):
    if six.PY2:
        parser = sp.add_parser('rebuild-results', help="Rebuild the html report from result dir")
    else:
        parser = sp.add_parser('rebuild-results', help="Rebuild the html report from result dir", aliases=['rebuild'])
    parser.add_argument('results_dir', help="The directory containing the results")
    parser.add_argument('results_file', help="The result file")
    parser.add_argument('config_file', help="The configuration file")
    parser.set_defaults(func=rebuild)



from __future__ import print_function
import os
import six
import sys
import shutil
import tarfile
from jinja2 import Environment, PackageLoader

BASE_DIR = os.path.abspath(os.path.dirname(__file__))


def check_template(members):

    required_members = [
        'config.json',
        'test_scripts',
        'templates',
        'templates/report.html'
    ]

    for fname in required_members:
        assert fname in members, "Required file %s not in archive" % fname

    return True


def from_template(args):
    """Create a new oct project from existing template

    :param Namespace args: command line arguments
    """
    project_name = args.name
    template = args.template

    with tarfile.open(template) as tar:
        check_template(tar.getnames())
        tar.extractall(project_name)


def from_oct(args):
    """Create a new oct project

    :param Namespace args: the command line arguments
    """
    project_name = args.name
    env = Environment(loader=PackageLoader('oct.utilities', 'templates'))

    config_content = env.get_template('configuration/config.json').render(script_name='v_user.py')
    script_content = env.get_template('scripts/v_user.j2').render()

    try:
        os.makedirs(project_name)
        os.makedirs(os.path.join(project_name, 'test_scripts'))
        os.makedirs(os.path.join(project_name, 'templates'))
        os.makedirs(os.path.join(project_name, 'templates', 'img'))

        shutil.copytree(os.path.join(BASE_DIR, 'templates', 'css'),
                        os.path.join(project_name, 'templates', 'css'))
        shutil.copytree(os.path.join(BASE_DIR, 'templates', 'javascript'),
                        os.path.join(project_name, 'templates', 'scripts'))
        shutil.copytree(os.path.join(BASE_DIR, 'templates', 'fonts'),
                        os.path.join(project_name, 'templates', 'fonts'))

        shutil.copy(os.path.join(BASE_DIR, 'templates', 'html', 'report.html'),
                    os.path.join(project_name, 'templates'))
    except OSError:
        print('ERROR: can not create directory for %r' % project_name, file=sys.stderr)
        raise
    with open(os.path.join(project_name, 'config.json'), 'w') as f:
        f.write(config_content)
    with open(os.path.join(project_name, 'test_scripts', 'v_user.py'), 'w') as f:
        f.write(script_content)


def create_project(args):

    if os.path.exists(args.name):
        raise OSError("Project %s already exists" % args.name)

    if args.template is not None:
        from_template(args)
    else:
        from_oct(args)


def new_project(sp):
    if six.PY2:
        parser = sp.add_parser('new-project', help="create a new oct project")
    else:
        parser = sp.add_parser('new-project', help="create a new oct project", aliases=['new'])
    parser.add_argument('name', type=str)
    parser.add_argument('-t', '--template', type=str, default=None, help="use existing project template")
    parser.set_defaults(func=create_project)
